#include <proto.h>

#include "gtlayer.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtLayer class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init layer.
*
\************************************************************************/
GtLayer *GtMkLayer(char *n,GtManager *p,GtTag *tgs)
{
	GtLayer *obj;
	
	if(!(obj=new GtLayer()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtLayer object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init layer.
*
\************************************************************************/
GtLayer *GtMkLayer(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtLayer *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkLayer(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty layer.
*
\************************************************************************/
GtLayer::GtLayer(void)
{
	curr=NULL;
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtLayer::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtLayer")==0)
		return(TRUE);
	else
		return(GtBdrMan::IsSubClassOf(cn));
}

/************************************************************************\
*
* Child asks if it is allowed to map or unmap itself.
*
\************************************************************************/
sbool GtLayer::MapReq(GtWidget *obj,sbool map)
{
	if(map)
	{
		if((obj!=curr) && curr)
			curr->Map(FALSE);
		curr=obj;
	}
	else
	{
		if(curr==obj)
			curr=NULL;
	}
	return(TRUE);
}

/************************************************************************\
*
* Relayout layer.
*
\************************************************************************/
void GtLayer::Relayout(void)
{
	GtWidget *obj;
	GtWListBdr *li;
	int xs,ys;
	word mxs,mys,mxxs=GTS_MAX,mxys=GTS_MAX;
	
	if(flags & GTWF_UNBUILT)
		return;
	mxs=1;
	mys=1;
	for(li=(GtWListBdr *)GtSFirst(&man); li;
			li=(GtWListBdr *)GtSNext(&li->wl.lnk))
	{
		obj=li->wl.w;
		obj->GetMinSize(&xs,&ys);
		mxs=GTMAX(mxs,xs+li->bleft+li->bright);
		mys=GTMAX(mys,ys+li->btop+li->bbot);
		obj->GetMaxSize(&xs,&ys);
		mxxs=GTMIN(mxxs,xs+li->bleft+li->bright);
		mxys=GTMIN(mxys,ys+li->btop+li->bbot);
	}
	mxxs=GTMAX(mxs,mxxs);
	mxys=GTMAX(mys,mxys);
	this->SetMinMaxSize(mxs,mys,mxxs,mxys);
}

/************************************************************************\
*
* Layout layer.
*
\************************************************************************/
void GtLayer::Layout(void)
{
	GtWListBdr *li;
	int xp,yp;
	
	if(flags & GTWF_UNBUILT)
		return;
	if(curr)
	{
		if(flags & GTWF_NOWIN)
		{
			xp=xpos;
			yp=ypos;
		}
		else
		{
			xp=0;
			yp=0;
		}
		li=(GtWListBdr *)curr->GetLInfo();
		curr->SetPosSize(xp+li->bleft,yp+li->btop,
			GTMIN(GTMAX(xsize,minxs),maxxs)-li->bleft-li->bright,
			GTMIN(GTMAX(ysize,minys),maxys)-li->btop-li->bbot);
	}
}
